import { HazelReader, HazelWriter, Vector2 } from "@skeldjs/util";
import { ExtractEventTypes } from "@skeldjs/events";
import { RoleTeamType, RoleType, SpawnType, SystemType } from "@skeldjs/constant";
import { BaseRpcMessage, CloseDoorsOfTypeMessage, RoleChanceSettings } from "@skeldjs/protocol";
import { AutoDoorsSystemEvents, DeconSystemEvents, DoorsSystemEvents, ElectricalDoorsSystemEvents, HqHudSystemEvents, HeliSabotageSystemEvents, LifeSuppSystemEvents, MedScanSystemEvents, MovingPlatformSystemEvents, ReactorSystemEvents, SabotageSystemEvents, SecurityCameraSystemEvents, SwitchSystemEvents, SystemStatus, SystemStatusEvents } from "../systems";
import { Networkable, NetworkableEvents } from "../Networkable";
import { Hostable } from "../Hostable";
import { PlayerData } from "../PlayerData";
import { BaseRole } from "../roles";
import { RoomAssignRolesEvent } from "../events";
export interface RoleAssignmentData {
    roleCtr: typeof BaseRole;
    chance: number;
    count: number;
}
declare type AllSystems = Map<SystemType, SystemStatus<any, any>>;
export interface ShipStatusData {
    systems: AllSystems;
}
export declare type ShipStatusEvents<RoomType extends Hostable = Hostable> = NetworkableEvents<RoomType> & DoorsSystemEvents<RoomType> & SystemStatusEvents<RoomType> & AutoDoorsSystemEvents<RoomType> & DeconSystemEvents<RoomType> & ElectricalDoorsSystemEvents<RoomType> & HqHudSystemEvents<RoomType> & HeliSabotageSystemEvents<RoomType> & LifeSuppSystemEvents<RoomType> & MedScanSystemEvents<RoomType> & MovingPlatformSystemEvents<RoomType> & ReactorSystemEvents<RoomType> & SabotageSystemEvents<RoomType> & SecurityCameraSystemEvents<RoomType> & SwitchSystemEvents<RoomType> & ExtractEventTypes<[RoomAssignRolesEvent<RoomType>]>;
export declare type ShipStatusType = SpawnType.SkeldShipStatus | SpawnType.MiraShipStatus | SpawnType.Polus | SpawnType.AprilShipStatus | SpawnType.Airship;
export declare class InnerShipStatus<RoomType extends Hostable = Hostable> extends Networkable<ShipStatusData, ShipStatusEvents<RoomType>, RoomType> {
    static roomDoors: Partial<Record<SystemType, number[]>>;
    systems: AllSystems;
    spawnRadius: number;
    initialSpawnCenter: Vector2;
    meetingSpawnCenter: Vector2;
    constructor(room: RoomType, spawnType: SpawnType, netId: number, ownerid: number, flags: number, data?: HazelReader | ShipStatusData);
    get owner(): RoomType;
    Setup(): void;
    Deserialize(reader: HazelReader, spawn?: boolean): void;
    Serialize(writer: HazelWriter, spawn?: boolean): boolean;
    HandleRpc(rpc: BaseRpcMessage): Promise<void>;
    protected _handleCloseDoorsOfType(rpc: CloseDoorsOfTypeMessage): Promise<void>;
    FixedUpdate(delta: number): void;
    private _handleRepairSystem;
    /**
     * Get all roles registered on the room that match a given filter.
     * @param filter The filter to match against.
     * @returns A list of all roles that satisfy the given filter.
     */
    matchRoles(settings: Partial<Record<RoleType, RoleChanceSettings>>, filter: (roleCtr: typeof BaseRole, roleChance: RoleChanceSettings) => any): (typeof BaseRole)[];
    /**
     * Specifically assign a pool of players to each role in a specific team.
     * @param playerPool The entire list of players that can be assigned a role,
     * whether or not they already have a role. Note that this is _not_ a list
     * of players to be assigned a role from this team, but it is instead every possible
     * player who _could_ be assigned one.
     * @param settings Role settings to use when calculating how many players
     * should assigned a specific role.
     * @param roleTeam The team of roles to assign.
     * @param maxAssignable The maximum number of players that can be assigned
     * a role from this team. For example, it could be the set number of impostors
     * configured in the game settings.
     * @param defaultRole The default role to assign to each player
     * @param roleAssignments A map of role assigments (i.e. a map of player to
     * roles) to act as a collective output to this method.
     * @returns The role assignments (i.e. a map of player to roles) that have
     * either collectively been assigned (if roleAssignments is passed)
     * or been assigned just as part of this method.
     */
    getRoleAssignmentsForTeam(playerPool: PlayerData[], settings: Partial<Record<RoleType, RoleChanceSettings>>, roleTeam: RoleTeamType, maxAssignable: number, defaultRole?: typeof BaseRole, roleAssignments?: Map<PlayerData, typeof BaseRole>): Map<PlayerData<Hostable<any>>, typeof BaseRole>;
    /**
     * Specifically assign a pool of players to each role in a list, with regard
     * to the room settings (i.e. the chance of each role appearing or the maximum
     * number of assignments for the role).
     * @param playerPool The entire list of players that can be assigned a role,
     * whether or not they already have a role. Note that this is _not_ a list
     * of players to be assigned a role from this team, but it is instead every possible
     * player who _could_ be assigned one.
     * @param settings Role settings to use when calculating how many players
     * should assigned a specific role.
     * @param roleTeam The team of roles to assign.
     * @param maxAssignable The maximum number of players that can be assigned
     * a role from this team. For example, it could be the set number of impostors
     * configured in the game settings.
     * @param defaultRole The default role to assign to each player
     * @param roleAssignments A map of role assigments (i.e. a map of player to
     * roles) to act as a collective output to this method.
     * @returns The role assignments (i.e. a map of player to roles) that have
     * either collectively been assigned (if roleAssignments is passed)
     * or been assigned just as part of this method.
     */
    getRoleAssignmentsFromRoleList(playerPool: PlayerData[], settings: Partial<Record<RoleType, RoleChanceSettings>>, teamRoles: typeof BaseRole[], maxAssignable: number, defaultRole?: typeof BaseRole, roleAssignments?: Map<PlayerData, typeof BaseRole>): Map<PlayerData<Hostable<any>>, typeof BaseRole>;
    /**
     * Assign a list of roles to a pool of players, eliminating both players
     * from the player list and also roles from the role list, and without regard
     * for room settings (i.e. the chances of each role appearing or the maximum
     * number of assignments for the role).
     * @param playerPool The pool of players to assign a list of roles to.
     * @param maxAssignable The maximum number of players that cn be assigned a
     * role from this list.
     * @param roleList The list of roles to assign to players.
     * @param roleAssignments A map of role assigments (i.e. a map of player to
     * roles) to act as a collective output to this method.
     * @returns The role assignments (i.e. a map of player to roles) that have
     * either collectively been assigned (if roleAssignments is passed)
     * or been assigned just as part of this method.
     */
    getRoleAssignmentsForPlayers(playerPool: PlayerData[], maxAssignable: number, roleList: typeof BaseRole[], roleAssignments?: Map<PlayerData, typeof BaseRole>): Map<PlayerData<Hostable<any>>, typeof BaseRole>;
    /**
     * Randomly assign players to each enabled role with a certain probability.
     * Called just after a game is started and emits a {@link RoomAssignRolesEvent}
     * which can be used to alter which players are assigned which roles.
     */
    assignRoles(): Promise<void>;
    /**
     * Try to assign a ghost role to a specific dead player.
     * @param player The player to assign the role to.
     */
    tryAssignGhostRole(player: PlayerData): Promise<void>;
    /**
     * Actually set all player roles from a map of player to role assignments,
     * can be gathered from {@link InnerShipStatus.getRoleAssignmentsForTeam}.
     * @param roleAssignments A map of player to role assignments to assign to
     * every role.
     */
    assignRolesFromAssignments(roleAssignments: Map<PlayerData, typeof BaseRole>): Promise<void>;
    private getTasksForMap;
    private addTasksFromList;
    /**
     * Randomly assign tasks to all players, using data from @skeldjs/data.
     */
    assignTasks(): void;
    /**
     * Get the spawn position of a player whether they are about to spawn after
     * starting or whether they are about to spawn after a meeting.
     * @param player The player or player ID to determine the position of.
     * @param initialSpawn Whther or not this is a spawn after starting the game.
     * @returns The spawn position of the player.
     */
    getSpawnPosition(player: PlayerData | number, initialSpawn: boolean): Vector2;
    /**
     * Teleport a player to their spawn position, calculated using
     * {@link InnerShipStatus.getSpawnPosition}.
     * @param player The player to determine the position of.
     * @param initialSpawn Whether or not this is a spawn after starting the game.
     */
    spawnPlayer(player: PlayerData, initialSpawn: boolean): void;
    /**
     * Get the door IDs used to connect to a room.
     * @param room The room to get the door IDs for.
     * @returns The door IDs that connect to the room.
     */
    getDoorsInRoom(room: SystemType): number[];
}
export {};
